@@ -6,9 +6,12 @@
 #include <stdarg.h>
 #include <pthread.h>
 #include <errno.h>
-#if HAVE_SCHED_GETAFFINITY
+#ifdef HAVE_SCHED_GETAFFINITY
 #include <sched.h>
 #endif
+#if defined(__linux__)
+#include <sys/personality.h>
+#endif
 
 #if !defined(isblank)
 #define	isblank(_c)	((_c) == ' ' || (_c) == '\t')
@@ -25,8 +28,8 @@
 #include <rpm/rpmmacro.h>
 #include <rpm/argv.h>
 
-#include "rpmio/rpmlua.h"
-#include "rpmio/rpmmacro_internal.h"
+#include "rpmlua.h"
+#include "rpmmacro_internal.h"
 #include "debug.h"
 
 enum macroFlags_e {
@@ -609,7 +612,7 @@ exit:
 static unsigned int getncpus(void)
 {
     unsigned int ncpus = 0;
-#if HAVE_SCHED_GETAFFINITY
+#ifdef HAVE_SCHED_GETAFFINITY
     cpu_set_t set;
     if (sched_getaffinity (0, sizeof(set), &set) == 0)
 	ncpus = CPU_COUNT(&set);
@@ -1105,6 +1108,30 @@ static void doLua(MacroBuf mb,  rpmMacroEntry me, ARGV_t argv, size_t *parsed)
     }
 }
 
+/*
+ * Wrap a call to Lua string functions.
+ * Note extra parentheses to force only one result returned, multi-value
+ * returns such as from string.gsub() make no sense in this context.
+ */
+static void doString(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
+{
+    rpmlua lua = NULL; /* Global state. */
+    char *printbuf = NULL;
+    char *s = rstrscat(NULL,
+		"return (string.", argv[0], "(table.unpack(arg)))", NULL);
+
+    rpmluaPushPrintBuffer(lua);
+    if (rpmluaRunScript(lua, s, argv[0], NULL, argv+1) == -1)
+	mb->error = 1;
+    printbuf = rpmluaPopPrintBuffer(lua);
+
+    if (printbuf) {
+	mbAppendStr(mb, printbuf);
+	free(printbuf);
+    }
+    free(s);
+}
+
 static void doSP(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
 {
     char *s = NULL;
@@ -1150,6 +1177,94 @@ static void doShescape(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parse
     mbAppend(mb, '\'');
 }
 
+static uint64_t getmem_total(void)
+{
+    uint64_t mem = 0;
+    long int pagesize = sysconf(_SC_PAGESIZE);
+    long int pages = sysconf(_SC_PHYS_PAGES);
+
+    if (pagesize <= 0)
+	pagesize = 4096;
+    if (pages > 0) {
+	/* Cast needed to force 64bit calculation on 32bit systems */
+	mem = (uint64_t)pages * pagesize;
+    }
+
+    return mem;
+}
+
+static uint64_t getmem_proc(int thread)
+{
+    uint64_t mem = getmem_total();
+    /*
+     * Conservative estimates for thread use on 32bit systems where address
+     * space is an issue: 2GB for bare metal, 3GB for a 32bit process
+     * on a 64bit system.
+     */
+    if (thread) {
+	uint64_t vmem = mem;
+#if __WORDSIZE == 32
+	vmem = UINT32_MAX / 2;
+#else
+#if defined(__linux__)
+	if ((personality(0xffffffff) & PER_MASK) == PER_LINUX32)
+	    vmem = (UINT32_MAX / 4) * 3;
+#endif
+#endif
+	if (vmem < mem)
+	    mem = vmem;
+    }
+    /* Fixup to get nice even numbers */
+    mem = mem / (1024*1024) + 1;
+
+    return mem;
+}
+
+static void doGetncpus(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
+{
+    const char *sizemacro = NULL;
+    const char *arg = (argv && argv[1]) ? argv[1] : "total";
+    char buf[32];
+    unsigned int ncpus = getncpus();
+    uint64_t mem = 0;
+
+    if (rstreq(arg, "total")) {
+	/* nothing */
+    } else if (rstreq(arg, "proc")) {
+	mem = getmem_proc(0);
+	sizemacro = "%{?_smp_tasksize_proc}";
+    } else if (rstreq(arg, "thread")) {
+	mem = getmem_proc(1);
+	sizemacro = "%{?_smp_tasksize_thread}";
+    } else {
+	mbErr(mb, 1, _("invalid argument: %s\n"), arg);
+	return;
+    }
+
+    if (sizemacro) {
+	unsigned int mcpus;
+	unsigned long tasksize = rpmExpandNumeric(sizemacro);
+
+	if (tasksize == 0)
+	    tasksize = 512;
+
+	if (mem == 0) {
+	    mbErr(mb, 1, _("failed to get available memory for %s\n"), arg);
+	    return;
+	}
+
+	mcpus = mem / tasksize;
+	if (mcpus < ncpus)
+	    ncpus = mcpus;
+    }
+    /* Ensure at least one CPU, no matter how starved */
+    if (ncpus < 1)
+	ncpus = 1;
+
+    sprintf(buf, "%u", ncpus);
+    mbAppendStr(mb, buf);
+}
+
 static void doFoo(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
 {
     char *buf = NULL;
@@ -1203,9 +1318,6 @@ static void doFoo(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
 	b = getenv(argv[1]);
     } else if (rstreq("getconfdir", me->name)) {
 	b = (char *)rpmConfigDir();
-    } else if (rstreq("getncpus", me->name)) {
-	b = buf = xmalloc(MACROBUFSIZ);
-	sprintf(buf, "%u", getncpus());
     } else if (rstreq("exists", me->name)) {
 	b = (access(argv[1], F_OK) == 0) ? "1" : "0";
     }
@@ -1257,20 +1369,27 @@ static struct builtins_s {
     { "expr",		doFoo,		1,	0 },
     { "getconfdir",	doFoo,		0,	0 },
     { "getenv",		doFoo,		1,	0 },
-    { "getncpus",	doFoo,		0,	0 },
+    { "getncpus",	doGetncpus,	-1,	0 },
     { "global",		doGlobal,	1,	ME_PARSE },
+    { "gsub",		doString,	1,	0 },
+    { "len",		doString,	1,	0 },
     { "load",		doLoad,		1,	0 },
+    { "lower",		doString,	1,	0 },
     { "lua",		doLua,		1,	ME_PARSE },
     { "macrobody",	doBody,		1,	0 },
     { "quote",		doFoo,		1,	0 },
+    { "rep",		doString,	1,	0 },
+    { "reverse",	doString,	1,	0 },
     { "rpmversion",	doRpmver,	0,	0 },
     { "shrink",		doFoo,		1,	0 },
+    { "sub",		doString,	1,	0 },
     { "suffix",		doFoo,		1,	0 },
     { "trace",		doTrace,	0,	0 },
     { "u2p",		doFoo,		1,	0 },
     { "shescape",	doShescape,	1,	0 },
     { "uncompress",	doUncompress,	1,	0 },
     { "undefine",	doUndefine,	1,	0 },
+    { "upper",		doString,	1,	0 },
     { "url2path",	doFoo,		1,	0 },
     { "verbose",	doVerbose,	-1,	ME_PARSE },
     { "warn",		doOutput,	1,	0 },
