@@ -11,9 +11,9 @@
 #include <rpm/rpmlog.h>
 #include <rpm/rpmbase64.h>
 
-#include "rpmio/rpmpgpval.h"
-#include "rpmio/rpmpgp_internal.h"
-#include "rpmio/rpmio_internal.h"	/* XXX rpmioSlurp */
+#include "rpmpgpval.h"
+#include "rpmpgp_internal.h"
+#include "rpmio_internal.h"	/* XXX rpmioSlurp */
 
 #include "debug.h"
 
@@ -47,14 +47,6 @@ struct pgpDigParams_s {
     pgpDigAlg alg;
 };
 
-/** \ingroup rpmio
- * Container for values parsed from an OpenPGP signature and public key.
- */
-struct pgpDig_s {
-    struct pgpDigParams_s * signature;
-    struct pgpDigParams_s * pubkey;
-};
-
 static void pgpPrtNL(void)
 {
     if (!_print) return;
@@ -815,13 +807,6 @@ static int pgpPrtPkt(struct pgpPkt *p, pgpDigParams _digp)
     return rc;
 }
 
-pgpDig pgpNewDig(void)
-{
-    pgpDig dig = xcalloc(1, sizeof(*dig));
-
-    return dig;
-}
-
 pgpDigParams pgpDigParamsFree(pgpDigParams digp)
 {
     if (digp) {
@@ -834,43 +819,6 @@ pgpDigParams pgpDigParamsFree(pgpDigParams digp)
     return NULL;
 }
 
-void pgpCleanDig(pgpDig dig)
-{
-    if (dig != NULL) {
-	pgpDigParamsFree(dig->signature);
-	pgpDigParamsFree(dig->pubkey);
-	memset(dig, 0, sizeof(*dig));
-    }
-    return;
-}
-
-pgpDig pgpFreeDig(pgpDig dig)
-{
-    if (dig != NULL) {
-
-	/* DUmp the signature/pubkey data. */
-	pgpCleanDig(dig);
-	dig = _free(dig);
-    }
-    return dig;
-}
-
-pgpDigParams pgpDigGetParams(pgpDig dig, unsigned int pkttype)
-{
-    pgpDigParams params = NULL;
-    if (dig) {
-	switch (pkttype) {
-	case PGPTAG_SIGNATURE:
-	    params = dig->signature;
-	    break;
-	case PGPTAG_PUBLIC_KEY:
-	    params = dig->pubkey;
-	    break;
-	}
-    }
-    return params;
-}
-
 int pgpDigParamsCmp(pgpDigParams p1, pgpDigParams p2)
 {
     int rc = 1; /* assume different, eg if either is NULL */
@@ -1095,6 +1043,14 @@ int pgpPrtParams(const uint8_t * pkts, size_t pktlen, unsigned int pkttype,
     return rc;
 }
 
+int pgpPrtParams2(const uint8_t * pkts, size_t pktlen, unsigned int pkttype,
+                  pgpDigParams * ret, char **lints)
+{
+    if (lints)
+        *lints = NULL;
+    return pgpPrtParams(pkts, pktlen, pkttype, ret);
+}
+
 int pgpPrtParamsSubkeys(const uint8_t *pkts, size_t pktlen,
 			pgpDigParams mainkey, pgpDigParams **subkeys,
 			int *subkeysCount)
@@ -1172,30 +1128,6 @@ int pgpPrtParamsSubkeys(const uint8_t *pkts, size_t pktlen,
     return rc;
 }
 
-int pgpPrtPkts(const uint8_t * pkts, size_t pktlen, pgpDig dig, int printing)
-{
-    int rc;
-    pgpDigParams digp = NULL;
-
-    _print = printing;
-
-    rc = pgpPrtParams(pkts, pktlen, 0, &digp);
-
-    if (dig && rc == 0) {
-	if (digp->tag == PGPTAG_SIGNATURE) {
-	    pgpDigParamsFree(dig->signature);
-	    dig->signature = digp;
-	} else {
-	    pgpDigParamsFree(dig->pubkey);
-	    dig->pubkey = digp;
-	}
-    } else {
-	pgpDigParamsFree(digp);
-    }
-
-    return rc;
-}
-
 rpmRC pgpVerifySignature(pgpDigParams key, pgpDigParams sig, DIGEST_CTX hashctx)
 {
     DIGEST_CTX ctx = rpmDigestDup(hashctx);
@@ -1255,13 +1187,11 @@ exit:
 
 }
 
-rpmRC pgpVerifySig(pgpDig dig, DIGEST_CTX hashctx)
+rpmRC pgpVerifySignature2(pgpDigParams key, pgpDigParams sig, DIGEST_CTX hashctx, char **lints)
 {
-    if (dig == NULL || hashctx == NULL)
-	return RPMRC_FAIL;
-
-    return pgpVerifySignature(pgpDigGetParams(dig, PGPTAG_PUBLIC_KEY),
-			      pgpDigGetParams(dig, PGPTAG_SIGNATURE), hashctx);
+    if (lints)
+        *lints = NULL;
+    return pgpVerifySignature(key, sig, hashctx);
 }
 
 static pgpArmor decodePkts(uint8_t *b, uint8_t **pkt, size_t *pktlen)
